@@ -16,9 +16,10 @@ import tornado.ioloop
 from tornado.web import StaticFileHandler
 from tornado.websocket import WebSocketHandler
 
-from . import utils
+from . import page
 from .remote_access import start_remote_access_service
-from .utils import make_applications, render_page, cdn_validation, deserialize_binary_event
+from .page import make_applications, render_page
+from .utils import cdn_validation, deserialize_binary_event, print_listen_address
 from ..session import CoroutineBasedSession, ThreadBasedSession, ScriptModeSession, \
     register_session_implement_for_target, Session
 from ..session.base import get_session_info_from_headers
@@ -65,7 +66,7 @@ def _is_same_site(origin, handler: WebSocketHandler):
     return origin == host
 
 
-def _webio_handler(applications=None, cdn=True, reconnect_timeout=0, check_origin_func=_is_same_site):
+def _webio_handler(applications=None, cdn=True, reconnect_timeout=0, check_origin_func=_is_same_site):  # noqa: C901
     """
     :param dict applications: dict of `name -> task function`
     :param bool/str cdn: Whether to load front-end static resources from CDN
@@ -93,6 +94,11 @@ def _webio_handler(applications=None, cdn=True, reconnect_timeout=0, check_origi
             app = applications.get(app_name) or applications['index']
             return app
 
+        def get_cdn(self):
+            if cdn is True and self.get_query_argument('_pywebio_cdn', '') == 'false':
+                return False
+            return cdn
+
         async def get(self, *args, **kwargs) -> None:
             # It's a simple http GET request
             if self.request.headers.get("Upgrade", "").lower() != "websocket":
@@ -102,7 +108,8 @@ def _webio_handler(applications=None, cdn=True, reconnect_timeout=0, check_origi
                     return self.write('')
 
                 app = self.get_app()
-                html = render_page(app, protocol='ws', cdn=cdn)
+
+                html = render_page(app, protocol='ws', cdn=self.get_cdn())
                 return self.write(html)
             else:
                 await super().get()
@@ -143,7 +150,12 @@ def _webio_handler(applications=None, cdn=True, reconnect_timeout=0, check_origi
                 return
 
             for msg in session.get_task_commands():
-                conn.write_message(json.dumps(msg))
+                try:
+                    conn.write_message(json.dumps(msg))
+                except TypeError as e:
+                    logger.exception('Data serialization error: %s\n'
+                                     'This may be because you pass the wrong type of parameter to the function'
+                                     ' of PyWebIO.\nData content: %s', e, msg)
 
         @classmethod
         def close_from_session(cls, session_id=None):
@@ -345,7 +357,7 @@ def start_server(applications, port=0, host='',
 
     cdn = cdn_validation(cdn, 'warn')  # if CDN is not available, warn user and disable CDN
 
-    utils.MAX_PAYLOAD_SIZE = max_payload_size = parse_file_size(max_payload_size)
+    page.MAX_PAYLOAD_SIZE = max_payload_size = parse_file_size(max_payload_size)
 
     debug = Session.debug = os.environ.get('PYWEBIO_DEBUG', debug)
 
@@ -361,7 +373,7 @@ def start_server(applications, port=0, host='',
     _, port = _setup_server(webio_handler=handler, port=port, host=host, static_dir=static_dir,
                             max_buffer_size=max_payload_size, **tornado_app_settings)
 
-    print('Listen on %s:%s' % (host or '0.0.0.0', port))
+    print_listen_address(host, port)
 
     if auto_open_webbrowser:
         tornado.ioloop.IOLoop.current().spawn_callback(open_webbrowser_on_server_started, host or 'localhost', port)
@@ -385,6 +397,7 @@ def start_server_in_current_thread_session():
     class SingleSessionWSHandler(_webio_handler(cdn=False)):
         session = None
         instance = None
+        closed = False
 
         def open(self):
             self.main_session = False
@@ -414,6 +427,7 @@ def start_server_in_current_thread_session():
         def on_close(self):
             if SingleSessionWSHandler.session is not None and self.main_session:
                 self.session.close()
+                self.closed = True
                 logger.debug('ScriptModeSession closed')
 
     async def wait_to_stop_loop(server):
@@ -427,7 +441,11 @@ def start_server_in_current_thread_session():
             alive_none_daemonic_thread_cnt = sum(
                 1 for t in threading.enumerate() if t.is_alive() and not t.isDaemon()
             )
-            await asyncio.sleep(1)
+            await asyncio.sleep(0.5)
+
+        if SingleSessionWSHandler.instance.session.need_keep_alive():
+            while not SingleSessionWSHandler.instance.closed:
+                await asyncio.sleep(0.5)
 
         # 关闭Websocket连接
         # Close the Websocket connection
@@ -462,7 +480,7 @@ def start_server_in_current_thread_session():
             port = int(os.environ.get("PYWEBIO_SCRIPT_MODE_PORT"))
 
         server, port = _setup_server(webio_handler=SingleSessionWSHandler, port=port, host='localhost',
-                                     websocket_max_message_size=parse_file_size('4G'))
+                                     websocket_max_message_size=parse_file_size('200M'))
         tornado.ioloop.IOLoop.current().spawn_callback(partial(wait_to_stop_loop, server=server))
 
         if "PYWEBIO_SCRIPT_MODE_PORT" not in os.environ:
